
#ifndef VTKTESTINGRENDERING_EXPORT_H
#define VTKTESTINGRENDERING_EXPORT_H

#ifdef VTKTESTINGRENDERING_STATIC_DEFINE
#  define VTKTESTINGRENDERING_EXPORT
#  define VTKTESTINGRENDERING_NO_EXPORT
#else
#  ifndef VTKTESTINGRENDERING_EXPORT
#    ifdef TestingRendering_EXPORTS
        /* We are building this library */
#      define VTKTESTINGRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKTESTINGRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKTESTINGRENDERING_NO_EXPORT
#    define VTKTESTINGRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKTESTINGRENDERING_DEPRECATED
#  define VTKTESTINGRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKTESTINGRENDERING_DEPRECATED_EXPORT
#  define VTKTESTINGRENDERING_DEPRECATED_EXPORT VTKTESTINGRENDERING_EXPORT VTKTESTINGRENDERING_DEPRECATED
#endif

#ifndef VTKTESTINGRENDERING_DEPRECATED_NO_EXPORT
#  define VTKTESTINGRENDERING_DEPRECATED_NO_EXPORT VTKTESTINGRENDERING_NO_EXPORT VTKTESTINGRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKTESTINGRENDERING_NO_DEPRECATED
#    define VTKTESTINGRENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKTESTINGRENDERING_EXPORT_H */
