#ifndef vtk_cgns_mangle_h
#define vtk_cgns_mangle_h

#define ADF_Children_IDs vtkcgns_ADF_Children_IDs
#define ADF_Children_Names vtkcgns_ADF_Children_Names
#define ADF_Create vtkcgns_ADF_Create
#define ADF_Database_Close vtkcgns_ADF_Database_Close
#define ADF_Database_Delete vtkcgns_ADF_Database_Delete
#define ADF_Database_Garbage_Collection vtkcgns_ADF_Database_Garbage_Collection
#define ADF_Database_Get_Format vtkcgns_ADF_Database_Get_Format
#define ADF_Database_Open vtkcgns_ADF_Database_Open
#define ADF_Database_Set_Format vtkcgns_ADF_Database_Set_Format
#define ADF_Database_Valid vtkcgns_ADF_Database_Valid
#define ADF_Database_Version vtkcgns_ADF_Database_Version
#define ADF_Delete vtkcgns_ADF_Delete
#define ADF_Error_Message vtkcgns_ADF_Error_Message
#define ADF_error_string vtkcgns_ADF_error_string
#define ADF_file vtkcgns_ADF_file
#define ADF_Flush_to_Disk vtkcgns_ADF_Flush_to_Disk
#define ADF_Get_Data_Type vtkcgns_ADF_Get_Data_Type
#define ADF_Get_Dimension_Values vtkcgns_ADF_Get_Dimension_Values
#define ADF_Get_Error_State vtkcgns_ADF_Get_Error_State
#define ADF_Get_Label vtkcgns_ADF_Get_Label
#define ADF_Get_Link_Path vtkcgns_ADF_Get_Link_Path
#define ADF_Get_Name vtkcgns_ADF_Get_Name
#define ADF_Get_Node_ID vtkcgns_ADF_Get_Node_ID
#define ADF_Get_Number_of_Dimensions vtkcgns_ADF_Get_Number_of_Dimensions
#define ADF_Get_Root_ID vtkcgns_ADF_Get_Root_ID
#define ADFH_Children_IDs vtkcgns_ADFH_Children_IDs
#define ADFH_Children_Names vtkcgns_ADFH_Children_Names
#define ADFH_Configure vtkcgns_ADFH_Configure
#define ADFH_Create vtkcgns_ADFH_Create
#define ADFH_Database_Close vtkcgns_ADFH_Database_Close
#define ADFH_Database_Delete vtkcgns_ADFH_Database_Delete
#define ADFH_Database_Garbage_Collection vtkcgns_ADFH_Database_Garbage_Collection
#define ADFH_Database_Get_Format vtkcgns_ADFH_Database_Get_Format
#define ADFH_Database_Open vtkcgns_ADFH_Database_Open
#define ADFH_Database_Set_Format vtkcgns_ADFH_Database_Set_Format
#define ADFH_Database_Valid vtkcgns_ADFH_Database_Valid
#define ADFH_Database_Version vtkcgns_ADFH_Database_Version
#define ADFH_Delete vtkcgns_ADFH_Delete
#define ADFH_Error_Message vtkcgns_ADFH_Error_Message
#define ADFH_Flush_to_Disk vtkcgns_ADFH_Flush_to_Disk
#define ADFH_Get_Data_Type vtkcgns_ADFH_Get_Data_Type
#define ADFH_Get_Dimension_Values vtkcgns_ADFH_Get_Dimension_Values
#define ADFH_Get_Error_State vtkcgns_ADFH_Get_Error_State
#define ADFH_Get_Label vtkcgns_ADFH_Get_Label
#define ADFH_Get_Link_Path vtkcgns_ADFH_Get_Link_Path
#define ADFH_Get_Name vtkcgns_ADFH_Get_Name
#define ADFH_Get_Node_ID vtkcgns_ADFH_Get_Node_ID
#define ADFH_Get_Number_of_Dimensions vtkcgns_ADFH_Get_Number_of_Dimensions
#define ADFH_Get_Root_ID vtkcgns_ADFH_Get_Root_ID
#define ADFH_Is_Link vtkcgns_ADFH_Is_Link
#define ADFH_Library_Version vtkcgns_ADFH_Library_Version
#define ADFH_Link vtkcgns_ADFH_Link
#define ADFH_Link_Size vtkcgns_ADFH_Link_Size
#define ADFH_Move_Child vtkcgns_ADFH_Move_Child
#define ADFH_Number_of_Children vtkcgns_ADFH_Number_of_Children
#define ADFH_Put_Dimension_Information vtkcgns_ADFH_Put_Dimension_Information
#define ADFH_Put_Name vtkcgns_ADFH_Put_Name
#define ADFH_Read_All_Data vtkcgns_ADFH_Read_All_Data
#define ADFH_Read_Block_Data vtkcgns_ADFH_Read_Block_Data
#define ADFH_Read_Data vtkcgns_ADFH_Read_Data
#define ADFH_Release_ID vtkcgns_ADFH_Release_ID
#define ADFH_Set_Error_State vtkcgns_ADFH_Set_Error_State
#define ADFH_Set_Label vtkcgns_ADFH_Set_Label
#define ADFH_Write_All_Data vtkcgns_ADFH_Write_All_Data
#define ADFH_Write_Block_Data vtkcgns_ADFH_Write_Block_Data
#define ADFH_Write_Data vtkcgns_ADFH_Write_Data
#define ADFI_Abort vtkcgns_ADFI_Abort
#define ADFI_add_2_sub_node_table vtkcgns_ADFI_add_2_sub_node_table
#define ADFI_adjust_disk_pointer vtkcgns_ADFI_adjust_disk_pointer
#define ADFI_ASCII_Hex_2_unsigned_int vtkcgns_ADFI_ASCII_Hex_2_unsigned_int
#define ADFI_big_endian_32_swap_64 vtkcgns_ADFI_big_endian_32_swap_64
#define ADFI_big_endian_to_cray vtkcgns_ADFI_big_endian_to_cray
#define ADFI_big_little_endian_swap vtkcgns_ADFI_big_little_endian_swap
#define ADFI_blank_fill_string vtkcgns_ADFI_blank_fill_string
#define ADFI_chase_link vtkcgns_ADFI_chase_link
#define ADFI_check_4_child_name vtkcgns_ADFI_check_4_child_name
#define ADFI_check_string_length vtkcgns_ADFI_check_string_length
#define ADFI_close_file vtkcgns_ADFI_close_file
#define ADFI_compare_node_names vtkcgns_ADFI_compare_node_names
#define ADFI_convert_number_format vtkcgns_ADFI_convert_number_format
#define ADFI_count_total_array_points vtkcgns_ADFI_count_total_array_points
#define ADFI_cray_to_big_endian vtkcgns_ADFI_cray_to_big_endian
#define ADFI_cray_to_little_endian vtkcgns_ADFI_cray_to_little_endian
#define ADFI_delete_data vtkcgns_ADFI_delete_data
#define ADFI_delete_from_sub_node_table vtkcgns_ADFI_delete_from_sub_node_table
#define ADFI_delete_sub_node_table vtkcgns_ADFI_delete_sub_node_table
#define ADFI_disk_pointer_2_ASCII_Hex vtkcgns_ADFI_disk_pointer_2_ASCII_Hex
#define ADFI_disk_pointer_from_ASCII_Hex vtkcgns_ADFI_disk_pointer_from_ASCII_Hex
#define ADFI_evaluate_datatype vtkcgns_ADFI_evaluate_datatype
#define ADFI_fflush_file vtkcgns_ADFI_fflush_file
#define ADFI_figure_machine_format vtkcgns_ADFI_figure_machine_format
#define ADFI_file_and_machine_compare vtkcgns_ADFI_file_and_machine_compare
#define ADFI_file_block_offset_2_ID vtkcgns_ADFI_file_block_offset_2_ID
#define ADFI_file_free vtkcgns_ADFI_file_free
#define ADFI_file_malloc vtkcgns_ADFI_file_malloc
#define ADFI_fill_initial_file_header vtkcgns_ADFI_fill_initial_file_header
#define ADFI_fill_initial_free_chunk_table vtkcgns_ADFI_fill_initial_free_chunk_table
#define ADFI_fill_initial_node_header vtkcgns_ADFI_fill_initial_node_header
#define ADFI_find_file vtkcgns_ADFI_find_file
#define ADFI_flush_buffers vtkcgns_ADFI_flush_buffers
#define ADFI_fseek_file vtkcgns_ADFI_fseek_file
#define ADFI_get_current_date vtkcgns_ADFI_get_current_date
#define ADFI_get_direct_children_ids vtkcgns_ADFI_get_direct_children_ids
#define ADFI_get_file_index_from_name vtkcgns_ADFI_get_file_index_from_name
#define ADFI_ID_2_file_block_offset vtkcgns_ADFI_ID_2_file_block_offset
#define ADFI_increment_array vtkcgns_ADFI_increment_array
#define ADFI_is_block_in_core vtkcgns_ADFI_is_block_in_core
#define ADFI_little_endian_32_swap_64 vtkcgns_ADFI_little_endian_32_swap_64
#define ADFI_little_endian_to_cray vtkcgns_ADFI_little_endian_to_cray
#define ADFI_open_file vtkcgns_ADFI_open_file
#define ADFI_read vtkcgns_ADFI_read
#define ADFI_read_chunk_length vtkcgns_ADFI_read_chunk_length
#define ADFI_read_data_chunk vtkcgns_ADFI_read_data_chunk
#define ADFI_read_data_chunk_table vtkcgns_ADFI_read_data_chunk_table
#define ADFI_read_data_translated vtkcgns_ADFI_read_data_translated
#define ADFI_read_disk_block vtkcgns_ADFI_read_disk_block
#define ADFI_read_disk_pointer vtkcgns_ADFI_read_disk_pointer
#define ADFI_read_disk_pointer_from_disk vtkcgns_ADFI_read_disk_pointer_from_disk
#define ADFI_read_file vtkcgns_ADFI_read_file
#define ADFI_read_file_header vtkcgns_ADFI_read_file_header
#define ADFI_read_free_chunk vtkcgns_ADFI_read_free_chunk
#define ADFI_read_free_chunk_table vtkcgns_ADFI_read_free_chunk_table
#define ADFI_read_node_header vtkcgns_ADFI_read_node_header
#define ADFI_read_sub_node_table vtkcgns_ADFI_read_sub_node_table
#define ADFI_read_sub_node_table_entry vtkcgns_ADFI_read_sub_node_table_entry
#define ADFI_remember_file_format vtkcgns_ADFI_remember_file_format
#define ADFI_remember_version_update vtkcgns_ADFI_remember_version_update
#define ADFI_set_blank_disk_pointer vtkcgns_ADFI_set_blank_disk_pointer
#define ADF_Is_Link vtkcgns_ADF_Is_Link
#define ADFI_stack_control vtkcgns_ADFI_stack_control
#define ADFI_stridx_c vtkcgns_ADFI_stridx_c
#define ADFI_string_2_C_string vtkcgns_ADFI_string_2_C_string
#define ADFI_strtok vtkcgns_ADFI_strtok
#define ADFI_unsigned_int_2_ASCII_Hex vtkcgns_ADFI_unsigned_int_2_ASCII_Hex
#define ADFI_write vtkcgns_ADFI_write
#define ADFI_write_data_chunk vtkcgns_ADFI_write_data_chunk
#define ADFI_write_data_chunk_table vtkcgns_ADFI_write_data_chunk_table
#define ADFI_write_data_translated vtkcgns_ADFI_write_data_translated
#define ADFI_write_disk_block vtkcgns_ADFI_write_disk_block
#define ADFI_write_disk_pointer vtkcgns_ADFI_write_disk_pointer
#define ADFI_write_disk_pointer_2_disk vtkcgns_ADFI_write_disk_pointer_2_disk
#define ADFI_write_file vtkcgns_ADFI_write_file
#define ADFI_write_file_header vtkcgns_ADFI_write_file_header
#define ADFI_write_free_chunk vtkcgns_ADFI_write_free_chunk
#define ADFI_write_free_chunk_table vtkcgns_ADFI_write_free_chunk_table
#define ADFI_write_modification_date vtkcgns_ADFI_write_modification_date
#define ADFI_write_node_header vtkcgns_ADFI_write_node_header
#define ADFI_write_sub_node_table vtkcgns_ADFI_write_sub_node_table
#define ADFI_write_sub_node_table_entry vtkcgns_ADFI_write_sub_node_table_entry
#define ADF_Library_Version vtkcgns_ADF_Library_Version
#define ADF_Link vtkcgns_ADF_Link
#define ADF_Link_Size vtkcgns_ADF_Link_Size
#define ADF_Move_Child vtkcgns_ADF_Move_Child
#define ADF_Number_of_Children vtkcgns_ADF_Number_of_Children
#define ADF_Put_Dimension_Information vtkcgns_ADF_Put_Dimension_Information
#define ADF_Put_Name vtkcgns_ADF_Put_Name
#define ADF_Read_All_Data vtkcgns_ADF_Read_All_Data
#define ADF_Read_Block_Data vtkcgns_ADF_Read_Block_Data
#define ADF_Read_Data vtkcgns_ADF_Read_Data
#define ADF_Set_Error_State vtkcgns_ADF_Set_Error_State
#define ADF_Set_Label vtkcgns_ADF_Set_Label
#define ADF_sys_err vtkcgns_ADF_sys_err
#define ADF_Write_All_Data vtkcgns_ADF_Write_All_Data
#define ADF_Write_Block_Data vtkcgns_ADF_Write_Block_Data
#define ADF_Write_Data vtkcgns_ADF_Write_Data
#define AngleUnitsName vtkcgns_AngleUnitsName
#define ArbitraryGridMotionTypeName vtkcgns_ArbitraryGridMotionTypeName
#define AreaTypeName vtkcgns_AreaTypeName
#define AverageInterfaceTypeName vtkcgns_AverageInterfaceTypeName
#define BCDataTypeName vtkcgns_BCDataTypeName
#define BCTypeName vtkcgns_BCTypeName
#define Cdim vtkcgns_Cdim
#define cg vtkcgns_cg
#define cg_1to1_average_read vtkcgns_cg_1to1_average_read
#define cg_1to1_average_write vtkcgns_cg_1to1_average_write
#define cg_1to1_id vtkcgns_cg_1to1_id
#define cg_1to1_periodic_read vtkcgns_cg_1to1_periodic_read
#define cg_1to1_periodic_write vtkcgns_cg_1to1_periodic_write
#define cg_1to1_read vtkcgns_cg_1to1_read
#define cg_1to1_read_global vtkcgns_cg_1to1_read_global
#define cg_1to1_write vtkcgns_cg_1to1_write
#define cg_add_path vtkcgns_cg_add_path
#define cg_AngleUnitsName vtkcgns_cg_AngleUnitsName
#define cg_ArbitraryGridMotionTypeName vtkcgns_cg_ArbitraryGridMotionTypeName
#define cg_arbitrary_motion_read vtkcgns_cg_arbitrary_motion_read
#define cg_arbitrary_motion_write vtkcgns_cg_arbitrary_motion_write
#define cg_AreaTypeName vtkcgns_cg_AreaTypeName
#define cg_array_general_read vtkcgns_cg_array_general_read
#define cg_array_general_write vtkcgns_cg_array_general_write
#define cg_array_info vtkcgns_cg_array_info
#define cg_array_read vtkcgns_cg_array_read
#define cg_array_read_as vtkcgns_cg_array_read_as
#define cg_array_write vtkcgns_cg_array_write
#define cg_AverageInterfaceTypeName vtkcgns_cg_AverageInterfaceTypeName
#define cg_axisym_read vtkcgns_cg_axisym_read
#define cg_axisym_write vtkcgns_cg_axisym_write
#define cg_base_id vtkcgns_cg_base_id
#define cg_base_read vtkcgns_cg_base_read
#define cg_base_write vtkcgns_cg_base_write
#define cg_bc_area_read vtkcgns_cg_bc_area_read
#define cg_bc_area_write vtkcgns_cg_bc_area_write
#define cg_bcdataset_info vtkcgns_cg_bcdataset_info
#define cg_bcdataset_read vtkcgns_cg_bcdataset_read
#define cg_bcdataset_write vtkcgns_cg_bcdataset_write
#define cg_BCDataTypeName vtkcgns_cg_BCDataTypeName
#define cg_bcdata_write vtkcgns_cg_bcdata_write
#define cg_BCTypeName vtkcgns_cg_BCTypeName
#define cg_bc_wallfunction_read vtkcgns_cg_bc_wallfunction_read
#define cg_bc_wallfunction_write vtkcgns_cg_bc_wallfunction_write
#define cg_biter_read vtkcgns_cg_biter_read
#define cg_biter_write vtkcgns_cg_biter_write
#define cg_boco_gridlocation_read vtkcgns_cg_boco_gridlocation_read
#define cg_boco_gridlocation_write vtkcgns_cg_boco_gridlocation_write
#define cg_boco_id vtkcgns_cg_boco_id
#define cg_boco_info vtkcgns_cg_boco_info
#define cg_boco_normal_write vtkcgns_cg_boco_normal_write
#define cg_boco_read vtkcgns_cg_boco_read
#define cg_boco_write vtkcgns_cg_boco_write
#define cg_cell_dim vtkcgns_cg_cell_dim
#define cg_close vtkcgns_cg_close
#define cg_configure vtkcgns_cg_configure
#define cg_conn_average_read vtkcgns_cg_conn_average_read
#define cg_conn_average_write vtkcgns_cg_conn_average_write
#define cg_conn_id vtkcgns_cg_conn_id
#define cg_conn_info vtkcgns_cg_conn_info
#define cg_conn_periodic_read vtkcgns_cg_conn_periodic_read
#define cg_conn_periodic_write vtkcgns_cg_conn_periodic_write
#define cg_conn_read vtkcgns_cg_conn_read
#define cg_conn_read_short vtkcgns_cg_conn_read_short
#define cg_conn_write vtkcgns_cg_conn_write
#define cg_conn_write_short vtkcgns_cg_conn_write_short
#define cg_convergence_read vtkcgns_cg_convergence_read
#define cg_convergence_write vtkcgns_cg_convergence_write
#define cg_conversion_info vtkcgns_cg_conversion_info
#define cg_conversion_read vtkcgns_cg_conversion_read
#define cg_conversion_write vtkcgns_cg_conversion_write
#define cg_coord_general_read vtkcgns_cg_coord_general_read
#define cg_coord_general_write vtkcgns_cg_coord_general_write
#define cg_coord_id vtkcgns_cg_coord_id
#define cg_coord_info vtkcgns_cg_coord_info
#define cg_coord_partial_write vtkcgns_cg_coord_partial_write
#define cg_coord_read vtkcgns_cg_coord_read
#define cg_coord_write vtkcgns_cg_coord_write
#define cg_DataClassName vtkcgns_cg_DataClassName
#define cg_dataclass_read vtkcgns_cg_dataclass_read
#define cg_dataclass_write vtkcgns_cg_dataclass_write
#define cg_dataset_read vtkcgns_cg_dataset_read
#define cg_dataset_write vtkcgns_cg_dataset_write
#define cg_DataTypeName vtkcgns_cg_DataTypeName
#define cg_delete_node vtkcgns_cg_delete_node
#define cg_descriptor_read vtkcgns_cg_descriptor_read
#define cg_descriptor_write vtkcgns_cg_descriptor_write
#define cg_diffusion_read vtkcgns_cg_diffusion_read
#define cg_diffusion_write vtkcgns_cg_diffusion_write
#define cg_discrete_ptset_info vtkcgns_cg_discrete_ptset_info
#define cg_discrete_ptset_read vtkcgns_cg_discrete_ptset_read
#define cg_discrete_ptset_write vtkcgns_cg_discrete_ptset_write
#define cg_discrete_read vtkcgns_cg_discrete_read
#define cg_discrete_size vtkcgns_cg_discrete_size
#define cg_discrete_write vtkcgns_cg_discrete_write
#define cg_ElectricCurrentUnitsName vtkcgns_cg_ElectricCurrentUnitsName
#define cg_ElementDataSize vtkcgns_cg_ElementDataSize
#define cg_ElementPartialSize vtkcgns_cg_ElementPartialSize
#define cg_elements_general_read vtkcgns_cg_elements_general_read
#define cg_elements_general_write vtkcgns_cg_elements_general_write
#define cg_elements_partial_read vtkcgns_cg_elements_partial_read
#define cg_elements_partial_write vtkcgns_cg_elements_partial_write
#define cg_elements_read vtkcgns_cg_elements_read
#define cg_ElementTypeName vtkcgns_cg_ElementTypeName
#define cg_equationset_chemistry_read vtkcgns_cg_equationset_chemistry_read
#define cg_equationset_elecmagn_read vtkcgns_cg_equationset_elecmagn_read
#define cg_equationset_read vtkcgns_cg_equationset_read
#define cg_equationset_write vtkcgns_cg_equationset_write
#define cg_error_exit vtkcgns_cg_error_exit
#define cg_error_handler vtkcgns_cg_error_handler
#define cg_error_print vtkcgns_cg_error_print
#define cg_expfull_read vtkcgns_cg_expfull_read
#define cg_expfull_write vtkcgns_cg_expfull_write
#define cg_exponents_info vtkcgns_cg_exponents_info
#define cg_exponents_read vtkcgns_cg_exponents_read
#define cg_exponents_write vtkcgns_cg_exponents_write
#define cg_fambc_read vtkcgns_cg_fambc_read
#define cg_fambc_write vtkcgns_cg_fambc_write
#define cg_family_name_read vtkcgns_cg_family_name_read
#define cg_family_name_write vtkcgns_cg_family_name_write
#define cg_family_read vtkcgns_cg_family_read
#define cg_family_write vtkcgns_cg_family_write
#define cg_famname_read vtkcgns_cg_famname_read
#define cg_famname_write vtkcgns_cg_famname_write
#define cg_field_general_read vtkcgns_cg_field_general_read
#define cg_field_general_write vtkcgns_cg_field_general_write
#define cg_field_id vtkcgns_cg_field_id
#define cg_field_info vtkcgns_cg_field_info
#define cg_field_partial_write vtkcgns_cg_field_partial_write
#define cg_field_read vtkcgns_cg_field_read
#define cg_field_write vtkcgns_cg_field_write
#define cg_free vtkcgns_cg_free
#define cg_geo_read vtkcgns_cg_geo_read
#define cg_geo_write vtkcgns_cg_geo_write
#define cg_get_cgio vtkcgns_cg_get_cgio
#define cg_get_compress vtkcgns_cg_get_compress
#define cg_get_error vtkcgns_cg_get_error
#define cg_get_file_type vtkcgns_cg_get_file_type
#define cg_get_name vtkcgns_cg_get_name
#define cg_golist vtkcgns_cg_golist
#define cg_gopath vtkcgns_cg_gopath
#define cg_gorel vtkcgns_cg_gorel
#define cg_gorel_f08 vtkcgns_cg_gorel_f08
#define cg_goto vtkcgns_cg_goto
#define cg_goto_f08 vtkcgns_cg_goto_f08
#define cg_GoverningEquationsTypeName vtkcgns_cg_GoverningEquationsTypeName
#define cg_governing_read vtkcgns_cg_governing_read
#define cg_governing_write vtkcgns_cg_governing_write
#define cg_gravity_read vtkcgns_cg_gravity_read
#define cg_gravity_write vtkcgns_cg_gravity_write
#define cg_grid_bounding_box_read vtkcgns_cg_grid_bounding_box_read
#define cg_grid_bounding_box_write vtkcgns_cg_grid_bounding_box_write
#define cg_GridConnectivityTypeName vtkcgns_cg_GridConnectivityTypeName
#define cg_GridLocationName vtkcgns_cg_GridLocationName
#define cg_gridlocation_read vtkcgns_cg_gridlocation_read
#define cg_gridlocation_write vtkcgns_cg_gridlocation_write
#define cg_grid_read vtkcgns_cg_grid_read
#define cg_grid_write vtkcgns_cg_grid_write
#define cg_hole_id vtkcgns_cg_hole_id
#define cg_hole_info vtkcgns_cg_hole_info
#define cg_hole_read vtkcgns_cg_hole_read
#define cg_hole_write vtkcgns_cg_hole_write
#define cgi_add_czone vtkcgns_cgi_add_czone
#define cgi_adf_datatype vtkcgns_cgi_adf_datatype
#define cgi_AngleUnits vtkcgns_cgi_AngleUnits
#define cgi_ArbitraryGridMotionType vtkcgns_cgi_ArbitraryGridMotionType
#define cgi_AreaType vtkcgns_cgi_AreaType
#define cgi_array_address vtkcgns_cgi_array_address
#define cgi_array_general_read vtkcgns_cgi_array_general_read
#define cgi_array_general_verify_range vtkcgns_cgi_array_general_verify_range
#define cgi_array_general_write vtkcgns_cgi_array_general_write
#define cgi_array_print vtkcgns_cgi_array_print
#define cgi_AverageInterfaceType vtkcgns_cgi_AverageInterfaceType
#define cgi_bcdataset_address vtkcgns_cgi_bcdataset_address
#define cgi_BCType vtkcgns_cgi_BCType
#define cgi_check_dimensions vtkcgns_cgi_check_dimensions
#define cgi_check_location vtkcgns_cgi_check_location
#define cgi_check_mode vtkcgns_cgi_check_mode
#define cgi_check_strlen vtkcgns_cgi_check_strlen
#define cgi_check_strlen_x2 vtkcgns_cgi_check_strlen_x2
#define cgi_converg_address vtkcgns_cgi_converg_address
#define cgi_conversion_address vtkcgns_cgi_conversion_address
#define cgi_convert_data vtkcgns_cgi_convert_data
#define cgi_DataClass vtkcgns_cgi_DataClass
#define cgi_dataclass_address vtkcgns_cgi_dataclass_address
#define cgi_datasize vtkcgns_cgi_datasize
#define cgi_datatype vtkcgns_cgi_datatype
#define cgi_delete_node vtkcgns_cgi_delete_node
#define cgi_descr_address vtkcgns_cgi_descr_address
#define cgi_diffusion_address vtkcgns_cgi_diffusion_address
#define cgi_ElectricCurrentUnits vtkcgns_cgi_ElectricCurrentUnits
#define cgi_element_data_size vtkcgns_cgi_element_data_size
#define cgi_equations_address vtkcgns_cgi_equations_address
#define cgi_error vtkcgns_cgi_error
#define cgi_exponent_address vtkcgns_cgi_exponent_address
#define cgi_family_address vtkcgns_cgi_family_address
#define cgi_famname_address vtkcgns_cgi_famname_address
#define cgi_free_1to1 vtkcgns_cgi_free_1to1
#define cgi_free_amotion vtkcgns_cgi_free_amotion
#define cgi_free_array vtkcgns_cgi_free_array
#define cgi_free_axisym vtkcgns_cgi_free_axisym
#define cgi_free_base vtkcgns_cgi_free_base
#define cgi_free_bcarea vtkcgns_cgi_free_bcarea
#define cgi_free_bcdata vtkcgns_cgi_free_bcdata
#define cgi_free_bcwall vtkcgns_cgi_free_bcwall
#define cgi_free_biter vtkcgns_cgi_free_biter
#define cgi_free_boco vtkcgns_cgi_free_boco
#define cgi_free_bprop vtkcgns_cgi_free_bprop
#define cgi_free_caverage vtkcgns_cgi_free_caverage
#define cgi_free_conn vtkcgns_cgi_free_conn
#define cgi_free_converg vtkcgns_cgi_free_converg
#define cgi_free_convert vtkcgns_cgi_free_convert
#define cgi_free_cperio vtkcgns_cgi_free_cperio
#define cgi_free_cprop vtkcgns_cgi_free_cprop
#define cgi_free_dataset vtkcgns_cgi_free_dataset
#define cgi_free_descr vtkcgns_cgi_free_descr
#define cgi_free_discrete vtkcgns_cgi_free_discrete
#define cgi_free_equations vtkcgns_cgi_free_equations
#define cgi_free_exponents vtkcgns_cgi_free_exponents
#define cgi_free_fambc vtkcgns_cgi_free_fambc
#define cgi_free_family vtkcgns_cgi_free_family
#define cgi_free_famname vtkcgns_cgi_free_famname
#define cgi_free_file vtkcgns_cgi_free_file
#define cgi_free_geo vtkcgns_cgi_free_geo
#define cgi_free_governing vtkcgns_cgi_free_governing
#define cgi_free_gravity vtkcgns_cgi_free_gravity
#define cgi_free_hole vtkcgns_cgi_free_hole
#define cgi_free_integral vtkcgns_cgi_free_integral
#define cgi_free_model vtkcgns_cgi_free_model
#define cgi_free_part vtkcgns_cgi_free_part
#define cgi_free_ptset vtkcgns_cgi_free_ptset
#define cgi_free_rmotion vtkcgns_cgi_free_rmotion
#define cgi_free_rotating vtkcgns_cgi_free_rotating
#define cgi_free_section vtkcgns_cgi_free_section
#define cgi_free_sol vtkcgns_cgi_free_sol
#define cgi_free_state vtkcgns_cgi_free_state
#define cgi_free_subreg vtkcgns_cgi_free_subreg
#define cgi_free_units vtkcgns_cgi_free_units
#define cgi_free_user_data vtkcgns_cgi_free_user_data
#define cgi_free_zboco vtkcgns_cgi_free_zboco
#define cgi_free_zconn vtkcgns_cgi_free_zconn
#define cgi_free_zcoor vtkcgns_cgi_free_zcoor
#define cgi_free_ziter vtkcgns_cgi_free_ziter
#define cgi_free_zone vtkcgns_cgi_free_zone
#define cgi_get_1to1 vtkcgns_cgi_get_1to1
#define cgi_get_amotion vtkcgns_cgi_get_amotion
#define cgi_get_axisym vtkcgns_cgi_get_axisym
#define cgi_get_base vtkcgns_cgi_get_base
#define cgi_get_bcdata vtkcgns_cgi_get_bcdata
#define cgi_get_biter vtkcgns_cgi_get_biter
#define cgi_get_boco vtkcgns_cgi_get_boco
#define cgi_get_bprop vtkcgns_cgi_get_bprop
#define cgi_get_conn vtkcgns_cgi_get_conn
#define cgi_get_converg vtkcgns_cgi_get_converg
#define cgi_get_cprop vtkcgns_cgi_get_cprop
#define cgi_get_dataset vtkcgns_cgi_get_dataset
#define cgi_get_discrete vtkcgns_cgi_get_discrete
#define cgi_get_equations vtkcgns_cgi_get_equations
#define cgi_get_family vtkcgns_cgi_get_family
#define cgi_get_field vtkcgns_cgi_get_field
#define cgi_get_file vtkcgns_cgi_get_file
#define cgi_get_governing vtkcgns_cgi_get_governing
#define cgi_get_gravity vtkcgns_cgi_get_gravity
#define cgi_get_hole vtkcgns_cgi_get_hole
#define cgi_get_integral vtkcgns_cgi_get_integral
#define cgi_get_model vtkcgns_cgi_get_model
#define cgi_get_nodes vtkcgns_cgi_get_nodes
#define cgi_get_posit vtkcgns_cgi_get_posit
#define cgi_get_rmotion vtkcgns_cgi_get_rmotion
#define cgi_get_rotating vtkcgns_cgi_get_rotating
#define cgi_get_section vtkcgns_cgi_get_section
#define cgi_get_sol vtkcgns_cgi_get_sol
#define cgi_get_state vtkcgns_cgi_get_state
#define cgi_get_subreg vtkcgns_cgi_get_subreg
#define cgi_get_zboco vtkcgns_cgi_get_zboco
#define cgi_get_zconn vtkcgns_cgi_get_zconn
#define cgi_get_zconnZC vtkcgns_cgi_get_zconnZC
#define cgi_get_zcoor vtkcgns_cgi_get_zcoor
#define cgi_get_zcoorGC vtkcgns_cgi_get_zcoorGC
#define cgi_get_ziter vtkcgns_cgi_get_ziter
#define cgi_get_zone vtkcgns_cgi_get_zone
#define cgi_governing_address vtkcgns_cgi_governing_address
#define cgi_GoverningEquationsType vtkcgns_cgi_GoverningEquationsType
#define cgi_GridConnectivityType vtkcgns_cgi_GridConnectivityType
#define cgi_GridLocation vtkcgns_cgi_GridLocation
#define cgi_hashmap_clear vtkcgns_cgi_hashmap_clear
#define cgi_integral_address vtkcgns_cgi_integral_address
#define cgi_LengthUnits vtkcgns_cgi_LengthUnits
#define cgi_location_address vtkcgns_cgi_location_address
#define cgi_LuminousIntensityUnits vtkcgns_cgi_LuminousIntensityUnits
#define cgi_malloc vtkcgns_cgi_malloc
#define cgi_map_contains vtkcgns_cgi_map_contains
#define cgi_map_del_shift_item vtkcgns_cgi_map_del_shift_item
#define cgi_map_get_item vtkcgns_cgi_map_get_item
#define cgi_map_set_item vtkcgns_cgi_map_set_item
#define cgi_MassUnits vtkcgns_cgi_MassUnits
#define cgi_model_address vtkcgns_cgi_model_address
#define cgi_ModelType vtkcgns_cgi_ModelType
#define cgi_move_node vtkcgns_cgi_move_node
#define cgi_multfam_address vtkcgns_cgi_multfam_address
#define cg_index_dim vtkcgns_cg_index_dim
#define cgi_new_hashmap vtkcgns_cgi_new_hashmap
#define cgi_new_presized_hashmap vtkcgns_cgi_new_presized_hashmap
#define cgi_new_node vtkcgns_cgi_new_node
#define cgi_new_node_partial vtkcgns_cgi_new_node_partial
#define cg_integral_read vtkcgns_cg_integral_read
#define cg_integral_write vtkcgns_cg_integral_write
#define cgio_check_dimensions vtkcgns_cgio_check_dimensions
#define cgio_check_file vtkcgns_cgio_check_file
#define cgio_children_ids vtkcgns_cgio_children_ids
#define cgio_children_names vtkcgns_cgio_children_names
#define cgio_cleanup vtkcgns_cgio_cleanup
#define cgio_close_file vtkcgns_cgio_close_file
#define cgio_compress_file vtkcgns_cgio_compress_file
#define cgio_compute_data_size vtkcgns_cgio_compute_data_size
#define cgio_configure vtkcgns_cgio_configure
#define cgio_copy_dimensions vtkcgns_cgio_copy_dimensions
#define cgio_copy_file vtkcgns_cgio_copy_file
#define cgio_copy_node vtkcgns_cgio_copy_node
#define cgio_create_link vtkcgns_cgio_create_link
#define cgio_create_node vtkcgns_cgio_create_node
#define cgio_delete_node vtkcgns_cgio_delete_node
#define cg_io_error vtkcgns_cg_io_error
#define cgio_error_abort vtkcgns_cgio_error_abort
#define cgio_error_code vtkcgns_cgio_error_code
#define cgio_error_exit vtkcgns_cgio_error_exit
#define cgio_error_message vtkcgns_cgio_error_message
#define cgio_file_version vtkcgns_cgio_file_version
#define cgio_find_file vtkcgns_cgio_find_file
#define cgio_flush_to_disk vtkcgns_cgio_flush_to_disk
#define cgio_get_data_size vtkcgns_cgio_get_data_size
#define cgio_get_data_type vtkcgns_cgio_get_data_type
#define cgio_get_dimensions vtkcgns_cgio_get_dimensions
#define cgio_get_file_type vtkcgns_cgio_get_file_type
#define cgio_get_label vtkcgns_cgio_get_label
#define cgio_get_link vtkcgns_cgio_get_link
#define cgio_get_name vtkcgns_cgio_get_name
#define cgio_get_node_id vtkcgns_cgio_get_node_id
#define cgio_get_root_id vtkcgns_cgio_get_root_id
#define cgio_is_link vtkcgns_cgio_is_link
#define cgio_is_supported vtkcgns_cgio_is_supported
#define cgio_library_version vtkcgns_cgio_library_version
#define cgio_link_size vtkcgns_cgio_link_size
#define cgio_move_node vtkcgns_cgio_move_node
#define cgio_new_node vtkcgns_cgio_new_node
#define cgio_number_children vtkcgns_cgio_number_children
#define cgio_open_file vtkcgns_cgio_open_file
#define cgio_path_add vtkcgns_cgio_path_add
#define cgio_path_delete vtkcgns_cgio_path_delete
#define cgi_ordinal_address vtkcgns_cgi_ordinal_address
#define cgio_read_all_data_type vtkcgns_cgio_read_all_data_type
#define cgio_read_block_data_type vtkcgns_cgio_read_block_data_type
#define cgio_read_data_type vtkcgns_cgio_read_data_type
#define cgio_release_id vtkcgns_cgio_release_id
#define cgio_set_dimensions vtkcgns_cgio_set_dimensions
#define cgio_set_label vtkcgns_cgio_set_label
#define cgio_set_name vtkcgns_cgio_set_name
#define cgio_write_all_data vtkcgns_cgio_write_all_data
#define cgio_write_all_data_type vtkcgns_cgio_write_all_data_type
#define cgio_write_block_data vtkcgns_cgio_write_block_data
#define cgio_write_data vtkcgns_cgio_write_data
#define cgio_write_data_type vtkcgns_cgio_write_data_type
#define cgi_PointSetType vtkcgns_cgi_PointSetType
#define cgi_posit_id vtkcgns_cgi_posit_id
#define cgi_posit_index_dim vtkcgns_cgi_posit_index_dim
#define cgi_ptset_address vtkcgns_cgi_ptset_address
#define cgi_read vtkcgns_cgi_read
#define cgi_read_1to1 vtkcgns_cgi_read_1to1
#define cgi_read_amotion vtkcgns_cgi_read_amotion
#define cgi_read_array vtkcgns_cgi_read_array
#define cgi_read_axisym vtkcgns_cgi_read_axisym
#define cgi_read_base vtkcgns_cgi_read_base
#define cgi_read_bcdata vtkcgns_cgi_read_bcdata
#define cgi_read_biter vtkcgns_cgi_read_biter
#define cgi_read_boco vtkcgns_cgi_read_boco
#define cgi_read_bprop vtkcgns_cgi_read_bprop
#define cgi_read_conn vtkcgns_cgi_read_conn
#define cgi_read_converg vtkcgns_cgi_read_converg
#define cgi_read_conversion vtkcgns_cgi_read_conversion
#define cgi_read_cprop vtkcgns_cgi_read_cprop
#define cgi_read_dataset vtkcgns_cgi_read_dataset
#define cgi_read_DDD vtkcgns_cgi_read_DDD
#define cgi_read_discrete vtkcgns_cgi_read_discrete
#define cgi_read_equations vtkcgns_cgi_read_equations
#define cgi_read_exponents vtkcgns_cgi_read_exponents
#define cgi_read_family vtkcgns_cgi_read_family
#define cgi_read_family_dataset vtkcgns_cgi_read_family_dataset
#define cgi_read_family_name vtkcgns_cgi_read_family_name
#define cgi_read_gravity vtkcgns_cgi_read_gravity
#define cgi_read_hole vtkcgns_cgi_read_hole
#define cgi_read_int_data vtkcgns_cgi_read_int_data
#define cgi_read_integral vtkcgns_cgi_read_integral
#define cgi_read_link vtkcgns_cgi_read_link
#define cgi_read_location vtkcgns_cgi_read_location
#define cgi_read_model vtkcgns_cgi_read_model
#define cgi_read_node vtkcgns_cgi_read_node
#define cgi_read_offset_data_type vtkcgns_cgi_read_offset_data_type
#define cgi_read_one_ptset vtkcgns_cgi_read_one_ptset
#define cgi_read_ordinal vtkcgns_cgi_read_ordinal
#define cgi_read_ptset vtkcgns_cgi_read_ptset
#define cgi_read_rind vtkcgns_cgi_read_rind
#define cgi_read_rmotion vtkcgns_cgi_read_rmotion
#define cgi_read_rotating vtkcgns_cgi_read_rotating
#define cgi_read_section vtkcgns_cgi_read_section
#define cgi_read_simulation vtkcgns_cgi_read_simulation
#define cgi_read_sol vtkcgns_cgi_read_sol
#define cgi_read_state vtkcgns_cgi_read_state
#define cgi_read_string vtkcgns_cgi_read_string
#define cgi_read_subregion vtkcgns_cgi_read_subregion
#define cgi_read_units vtkcgns_cgi_read_units
#define cgi_read_user_data vtkcgns_cgi_read_user_data
#define cgi_read_zboco vtkcgns_cgi_read_zboco
#define cgi_read_zconn vtkcgns_cgi_read_zconn
#define cgi_read_zcoor vtkcgns_cgi_read_zcoor
#define cgi_read_ziter vtkcgns_cgi_read_ziter
#define cgi_read_zone vtkcgns_cgi_read_zone
#define cgi_read_zonetype vtkcgns_cgi_read_zonetype
#define cgi_realloc vtkcgns_cgi_realloc
#define cgi_RigidGridMotionType vtkcgns_cgi_RigidGridMotionType
#define cgi_rind_address vtkcgns_cgi_rind_address
#define cgi_rotating_address vtkcgns_cgi_rotating_address
#define cg_is_cgns vtkcgns_cg_is_cgns
#define cgi_set_posit vtkcgns_cgi_set_posit
#define cgi_SimulationType vtkcgns_cgi_SimulationType
#define cg_is_link vtkcgns_cg_is_link
#define cgi_sort_names vtkcgns_cgi_sort_names
#define cgi_state_address vtkcgns_cgi_state_address
#define cgi_SubstanceAmountUnits vtkcgns_cgi_SubstanceAmountUnits
#define cgi_TemperatureUnits vtkcgns_cgi_TemperatureUnits
#define cgi_TimeUnits vtkcgns_cgi_TimeUnits
#define cgi_units_address vtkcgns_cgi_units_address
#define cgi_update_posit vtkcgns_cgi_update_posit
#define cgi_user_data_address vtkcgns_cgi_user_data_address
#define cgi_WallFunctionType vtkcgns_cgi_WallFunctionType
#define cgi_warning vtkcgns_cgi_warning
#define cgi_write vtkcgns_cgi_write
#define cgi_write_1to1 vtkcgns_cgi_write_1to1
#define cgi_write_amotion vtkcgns_cgi_write_amotion
#define cgi_write_array vtkcgns_cgi_write_array
#define cgi_write_axisym vtkcgns_cgi_write_axisym
#define cgi_write_bcdata vtkcgns_cgi_write_bcdata
#define cgi_write_biter vtkcgns_cgi_write_biter
#define cgi_write_boco vtkcgns_cgi_write_boco
#define cgi_write_bprop vtkcgns_cgi_write_bprop
#define cgi_write_conns vtkcgns_cgi_write_conns
#define cgi_write_converg vtkcgns_cgi_write_converg
#define cgi_write_cprop vtkcgns_cgi_write_cprop
#define cgi_write_dataclass vtkcgns_cgi_write_dataclass
#define cgi_write_dataset vtkcgns_cgi_write_dataset
#define cgi_write_descr vtkcgns_cgi_write_descr
#define cgi_write_discrete vtkcgns_cgi_write_discrete
#define cgi_write_equations vtkcgns_cgi_write_equations
#define cgi_write_exponents vtkcgns_cgi_write_exponents
#define cgi_write_family vtkcgns_cgi_write_family
#define cgi_write_gravity vtkcgns_cgi_write_gravity
#define cgi_write_holes vtkcgns_cgi_write_holes
#define cgi_write_integral vtkcgns_cgi_write_integral
#define cgi_write_link vtkcgns_cgi_write_link
#define cgi_write_model vtkcgns_cgi_write_model
#define cgi_write_ordinal vtkcgns_cgi_write_ordinal
#define cgi_write_ptset vtkcgns_cgi_write_ptset
#define cgi_write_rind vtkcgns_cgi_write_rind
#define cgi_write_rmotion vtkcgns_cgi_write_rmotion
#define cgi_write_rotating vtkcgns_cgi_write_rotating
#define cgi_write_section vtkcgns_cgi_write_section
#define cgi_write_sol vtkcgns_cgi_write_sol
#define cgi_write_state vtkcgns_cgi_write_state
#define cgi_write_units vtkcgns_cgi_write_units
#define cgi_write_user_data vtkcgns_cgi_write_user_data
#define cgi_write_zboco vtkcgns_cgi_write_zboco
#define cgi_write_zconn vtkcgns_cgi_write_zconn
#define cgi_write_zcoor vtkcgns_cgi_write_zcoor
#define cgi_write_ziter vtkcgns_cgi_write_ziter
#define cgi_write_zone vtkcgns_cgi_write_zone
#define cgi_zone_no vtkcgns_cgi_zone_no
#define cgi_ZoneType vtkcgns_cgi_ZoneType
#define cg_LengthUnitsName vtkcgns_cg_LengthUnitsName
#define cg_link_read vtkcgns_cg_link_read
#define cg_link_write vtkcgns_cg_link_write
#define cg_LuminousIntensityUnitsName vtkcgns_cg_LuminousIntensityUnitsName
#define cg_MassUnitsName vtkcgns_cg_MassUnitsName
#define cg_model_read vtkcgns_cg_model_read
#define cg_ModelTypeName vtkcgns_cg_ModelTypeName
#define cg_model_write vtkcgns_cg_model_write
#define cg_multifam_read vtkcgns_cg_multifam_read
#define cg_multifam_write vtkcgns_cg_multifam_write
#define cg_n1to1 vtkcgns_cg_n1to1
#define cg_n1to1_global vtkcgns_cg_n1to1_global
#define cg_n_arbitrary_motions vtkcgns_cg_n_arbitrary_motions
#define cg_narrays vtkcgns_cg_narrays
#define cg_nbases vtkcgns_cg_nbases
#define cg_nbocos vtkcgns_cg_nbocos
#define cg_nconns vtkcgns_cg_nconns
#define cg_ncoords vtkcgns_cg_ncoords
#define cg_ndescriptors vtkcgns_cg_ndescriptors
#define cg_ndiscrete vtkcgns_cg_ndiscrete
#define cg_nexponents vtkcgns_cg_nexponents
#define cg_nfamilies vtkcgns_cg_nfamilies
#define cg_nfamily_names vtkcgns_cg_nfamily_names
#define cg_nfields vtkcgns_cg_nfields
#define cg_ngrids vtkcgns_cg_ngrids
#define cg_nholes vtkcgns_cg_nholes
#define cg_nintegrals vtkcgns_cg_nintegrals
#define cg_nmultifam vtkcgns_cg_nmultifam
#define cg_node_fambc_read vtkcgns_cg_node_fambc_read
#define cg_node_fambc_write vtkcgns_cg_node_fambc_write
#define cg_node_family_name_read vtkcgns_cg_node_family_name_read
#define cg_node_family_name_write vtkcgns_cg_node_family_name_write
#define cg_node_family_read vtkcgns_cg_node_family_read
#define cg_node_family_write vtkcgns_cg_node_family_write
#define cg_node_geo_read vtkcgns_cg_node_geo_read
#define cg_node_geo_write vtkcgns_cg_node_geo_write
#define cg_node_nfamilies vtkcgns_cg_node_nfamilies
#define cg_node_nfamily_names vtkcgns_cg_node_nfamily_names
#define cg_node_part_read vtkcgns_cg_node_part_read
#define cg_node_part_write vtkcgns_cg_node_part_write
#define cg_npe vtkcgns_cg_npe
#define cg_n_rigid_motions vtkcgns_cg_n_rigid_motions
#define cgns_compress vtkcgns_cgns_compress
#define cg_nsections vtkcgns_cg_nsections
#define cgns_error_handler vtkcgns_cgns_error_handler
#define cgns_error_mess vtkcgns_cgns_error_mess
#define cgns_files vtkcgns_cgns_files
#define cgns_file_size vtkcgns_cgns_file_size
#define cgns_filetype vtkcgns_cgns_filetype
#define CGNSLibVersion vtkcgns_CGNSLibVersion
#define cg_nsols vtkcgns_cg_nsols
#define cgns_rindindex vtkcgns_cgns_rindindex
#define cg_nsubregs vtkcgns_cg_nsubregs
#define cg_nunits vtkcgns_cg_nunits
#define cg_nuser_data vtkcgns_cg_nuser_data
#define cg_nzconns vtkcgns_cg_nzconns
#define cg_nzones vtkcgns_cg_nzones
#define cg_open vtkcgns_cg_open
#define cg_ordinal_read vtkcgns_cg_ordinal_read
#define cg_ordinal_write vtkcgns_cg_ordinal_write
#define cg_parent_data_partial_write vtkcgns_cg_parent_data_partial_write
#define cg_parent_data_write vtkcgns_cg_parent_data_write
#define cg_parent_elements_general_read vtkcgns_cg_parent_data_elements_general_read
#define cg_parent_elements_position_general_read vtkcgns_cg_parent_data_elements_position_general_read
#define cg_part_read vtkcgns_cg_part_read
#define cg_part_write vtkcgns_cg_part_write
#define cg_PointSetTypeName vtkcgns_cg_PointSetTypeName
#define cg_poly_elements_general_read vtkcgns_cg_poly_elements_general_read
#define cg_poly_elements_general_write vtkcgns_cg_poly_elements_general_write
#define cg_poly_elements_partial_read vtkcgns_cg_poly_elements_partial_read
#define cg_poly_elements_partial_write vtkcgns_cg_poly_elements_partial_write
#define cg_poly_elements_read vtkcgns_cg_poly_elements_read
#define cg_poly_section_write vtkcgns_cg_poly_section_write
#define cg_precision vtkcgns_cg_precision
#define cg_ptset_info vtkcgns_cg_ptset_info
#define cg_ptset_read vtkcgns_cg_ptset_read
#define cg_ptset_write vtkcgns_cg_ptset_write
#define cg_RigidGridMotionTypeName vtkcgns_cg_RigidGridMotionTypeName
#define cg_rigid_motion_read vtkcgns_cg_rigid_motion_read
#define cg_rigid_motion_write vtkcgns_cg_rigid_motion_write
#define cg_rind_read vtkcgns_cg_rind_read
#define cg_rind_write vtkcgns_cg_rind_write
#define cg_root_id vtkcgns_cg_root_id
#define cg_rotating_read vtkcgns_cg_rotating_read
#define cg_rotating_write vtkcgns_cg_rotating_write
#define cg_save_as vtkcgns_cg_save_as
#define cg_section_general_write vtkcgns_cg_section_general_write
#define cg_section_initialize vtkcgns_cg_section_initialize
#define cg_section_partial_write vtkcgns_cg_section_partial_write
#define cg_section_read vtkcgns_cg_section_read
#define cg_section_write vtkcgns_cg_section_write
#define cg_set_compress vtkcgns_cg_set_compress
#define cg_set_file_type vtkcgns_cg_set_file_type
#define cg_set_path vtkcgns_cg_set_path
#define cg_SimulationTypeName vtkcgns_cg_SimulationTypeName
#define cg_simulation_type_read vtkcgns_cg_simulation_type_read
#define cg_simulation_type_write vtkcgns_cg_simulation_type_write
#define cg_sol_id vtkcgns_cg_sol_id
#define cg_sol_info vtkcgns_cg_sol_info
#define cg_sol_ptset_info vtkcgns_cg_sol_ptset_info
#define cg_sol_ptset_read vtkcgns_cg_sol_ptset_read
#define cg_sol_ptset_write vtkcgns_cg_sol_ptset_write
#define cg_sol_size vtkcgns_cg_sol_size
#define cg_sol_write vtkcgns_cg_sol_write
#define cg_state_read vtkcgns_cg_state_read
#define cg_state_write vtkcgns_cg_state_write
#define cg_subreg_bcname_read vtkcgns_cg_subreg_bcname_read
#define cg_subreg_bcname_write vtkcgns_cg_subreg_bcname_write
#define cg_subreg_gcname_read vtkcgns_cg_subreg_gcname_read
#define cg_subreg_gcname_write vtkcgns_cg_subreg_gcname_write
#define cg_subreg_info vtkcgns_cg_subreg_info
#define cg_subreg_ptset_read vtkcgns_cg_subreg_ptset_read
#define cg_subreg_ptset_write vtkcgns_cg_subreg_ptset_write
#define cg_SubstanceAmountUnitsName vtkcgns_cg_SubstanceAmountUnitsName
#define cg_TemperatureUnitsName vtkcgns_cg_TemperatureUnitsName
#define cg_TimeUnitsName vtkcgns_cg_TimeUnitsName
#define cg_unitsfull_read vtkcgns_cg_unitsfull_read
#define cg_unitsfull_write vtkcgns_cg_unitsfull_write
#define cg_units_read vtkcgns_cg_units_read
#define cg_units_write vtkcgns_cg_units_write
#define cg_user_data_read vtkcgns_cg_user_data_read
#define cg_user_data_write vtkcgns_cg_user_data_write
#define cg_version vtkcgns_cg_version
#define cg_WallFunctionTypeName vtkcgns_cg_WallFunctionTypeName
#define cg_where vtkcgns_cg_where
#define cg_zconn_get vtkcgns_cg_zconn_get
#define cg_zconn_read vtkcgns_cg_zconn_read
#define cg_zconn_set vtkcgns_cg_zconn_set
#define cg_zconn_write vtkcgns_cg_zconn_write
#define cg_ziter_read vtkcgns_cg_ziter_read
#define cg_ziter_write vtkcgns_cg_ziter_write
#define cg_zone_id vtkcgns_cg_zone_id
#define cg_zone_read vtkcgns_cg_zone_read
#define cg_zone_type vtkcgns_cg_zone_type
#define cg_ZoneTypeName vtkcgns_cg_ZoneTypeName
#define cg_zone_write vtkcgns_cg_zone_write
#define CurrentDim vtkcgns_CurrentDim
#define CurrentZoneType vtkcgns_CurrentZoneType
#define data_chunk_start_tag vtkcgns_data_chunk_start_tag
#define DataClassName vtkcgns_DataClassName
#define DataTypeName vtkcgns_DataTypeName
#define ElectricCurrentUnitsName vtkcgns_ElectricCurrentUnitsName
#define ElementTypeName vtkcgns_ElementTypeName
#define file_number_offset vtkcgns_file_number_offset
#define GoverningEquationsTypeName vtkcgns_GoverningEquationsTypeName
#define GridConnectivityTypeName vtkcgns_GridConnectivityTypeName
#define GridLocationName vtkcgns_GridLocationName
#define hdf5_access vtkcgns_hdf5_access
#define HDF5storage_type vtkcgns_HDF5storage_type
#define Idim vtkcgns_Idim
#define LengthUnitsName vtkcgns_LengthUnitsName
#define LuminousIntensityUnitsName vtkcgns_LuminousIntensityUnitsName
#define MassUnitsName vtkcgns_MassUnitsName
#define maximum_files vtkcgns_maximum_files
#define ModelTypeName vtkcgns_ModelTypeName
#define n_cgns_files vtkcgns_n_cgns_files
#define n_open vtkcgns_n_open
#define NumberOfSteps vtkcgns_NumberOfSteps
#define Pdim vtkcgns_Pdim
#define PointSetTypeName vtkcgns_PointSetTypeName
#define posit vtkcgns_posit
#define posit_base vtkcgns_posit_base
#define posit_depth vtkcgns_posit_depth
#define posit_file vtkcgns_posit_file
#define posit_stack vtkcgns_posit_stack
#define posit_zone vtkcgns_posit_zone
#define RigidGridMotionTypeName vtkcgns_RigidGridMotionTypeName
#define SimulationTypeName vtkcgns_SimulationTypeName
#define size_of vtkcgns_size_of
#define SubstanceAmountUnitsName vtkcgns_SubstanceAmountUnitsName
#define TemperatureUnitsName vtkcgns_TemperatureUnitsName
#define TimeUnitsName vtkcgns_TimeUnitsName
#define type_of vtkcgns_type_of
#define vcg_gorel vtkcgns_vcg_gorel
#define vcg_goto vtkcgns_vcg_goto
#define VersionList vtkcgns_VersionList
#define WallFunctionTypeName vtkcgns_WallFunctionTypeName
#define ZoneTypeName vtkcgns_ZoneTypeName

#endif
